;------------------------------------------------------------------------------
;  LTC1662.asm
; 
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2010-2013 all rights reserved
;
;  - Handle the LTC1661/1662 10-bit D/A converter
;  - Implement all control functions
;  - Use the PIC16F SPI port
;  - One additional control line is required for Load...
;    daPort= port address for control line
;    daDir= port direction control address
;    daLoad= bit position for load line on daPort
;  - DAInit MUST be called before any other routine
;
;  30Jul2010 A. Cooper
;  - Initial version
;  21Aug2010 A. Cooper
;  - Re-written for two DAC's and setting voltage only (no sleep)
;  31Jan2013 A. Cooper
;  - removed second DAC
;  - converted for use with PIC18F873 (non A version)
;
;------------------------------------------------------------------------------


;------------------------------------------------------------------------------
;  DAInit
;  - Initialize the LTC1662 D/A converter
;  - setup the SPI peripheral
;  - set the D/A inactive and voltage to zero
;  - SCLK must be inactive low (must be low when Load is asserted on LTC1662)
;------------------------------------------------------------------------------
DAInit	movlw	b'11000000'	;configure SPI
	banksel	SSPSTAT
	movwf	SSPSTAT
	movlw	b'00100000'
	banksel	SSPCON
	movwf	SSPCON
	banksel	TRISC	;set port pin directions
	bcf	TRISC,5	;SDO set for output
	bcf	TRISC,3	;SCK set for output
	pagesel	DAZero	;insure outputs zero'd
	call	DAZero
	banksel	daDir	;set control line
	bcf	daDir,daLoad
	banksel	daPort
	bsf	daPort,daLoad
	return


;------------------------------------------------------------------------------
;  DASet
;  - Set the voltage for output the commanded channel
;  - Channel in W range 0-1
;  - 10bit voltage in paramH and paramL
;  - DAC outputs enabled after call
;------------------------------------------------------------------------------
DASet	andlw	0x01	;insure legal channel 0 or 1
	pagesel	DAChan0
	btfsc	STATUS,Z
	 goto	DAChan0
	pagesel	DAChan1
	goto	DAChan1
	return


;------------------------------------------------------------------------------
;  DAZero
;  - Zero all DAC channels
;------------------------------------------------------------------------------
DAZero	
	banksel	paramH	;clear setting
	clrf	paramH
	clrf	paramL
	pagesel	DAChan0
	call	DAChan0
	banksel	paramH	
	clrf	paramH
	clrf	paramL
	pagesel	DAChan1
	call	DAChan1
	return


;------------------------------------------------------------------------------
;  DAChanX
;  - Send the correct commands and load bits to set a channel
;
;------------------------------------------------------------------------------
DAChan0	movlw	0x03	;mask lower two bits of paramH
	banksel	paramH
	andwf	paramH,f
	banksel	daPort	;assert load low
	bcf	daPort,daLoad
	movlw	0x09	;send load DAC A Command
	pagesel	DACmd
	call	DACmd
	banksel	daPort	;de-assert load
	bsf	daPort,daLoad
	return

DAChan1	movlw	0x03	;mask lower two bits of paramH
	banksel	paramH
	andwf	paramH,f
	banksel	daPort	;assert load low
	bcf	daPort,daLoad
	movlw	0x0A	;send load DAC B Command
	pagesel	DACmd
	call	DACmd
	banksel	daPort	;de-assert load
	bsf	daPort,daLoad
	return


;------------------------------------------------------------------------------
;  DACmd
;  - Send the specified command to the DAC
;  - command in lower nibble of W
;  - operand in paramH and paramL, right justified
;------------------------------------------------------------------------------
DACmd	andlw	0x0F	;insure command 4-bits
	banksel	temp
	movwf	temp	;temp store command
	swapf	paramH,f	;get upper bits of high byte
	rlf	paramH,f
	rlf	paramH,f
	rlf	paramH,f
	rlf	temp,f
	rlf	paramH,f
	rlf	temp,f
	rlf	paramL,f
	rlf	temp,f
	rlf	paramL,f
	rlf	temp,w
	banksel	PIR1	;setup transfer
	bcf	PIR1,SSPIF
	banksel	SSPBUF
	movwf	SSPBUF	;send to serial
	banksel	PIR1
DACmd1	btfss	PIR1,SSPIF
	 goto	DACmd1
	banksel	paramL
	movf	paramL,w	;send second byte
	banksel	PIR1
	bcf	PIR1,SSPIF
	banksel	SSPBUF
	movwf	SSPBUF
	banksel	PIR1
DACmd2	btfss	PIR1,SSPIF
	 goto	DACmd2
	return


; end of LTC1662.asm
;----------------------------------------------------------------------------
