;------------------------------------------------------------------------------
;  iSource.asm
;  - Programmable 1A current source
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2003-2013 all rights reserved
;
;  26Feb2010 A. Cooper
;  - initial version, based on LEDCtrl
;
;  To Do:
;  - Add LTC1661 DAC				<-Done
;  - Re-write command set			<-Done
;  - Power to DAC setting method	<-Done
;  - Current readback with ADC 		<-Done
;  - Routine reset on timeout		<-Done
;  - Insure no lamp flicker on init	<-Done
;  - Acceptance testing 			<-Done
;
;------------------------------------------------------------------------------
	title	"ISource"
	LIST	P=PIC16F873
	include	"PIC16F873.inc"

;------------------------------------------------------------------------------
; make for debug or stand alone
#define	debug	1
	ifdef	debug
	__config	B'11011100110001'	;debugger operation
	else
	__config	B'11111101110101'	;std operation
	endif

            errorlevel  -302, -306

;clock calibration constant
clockCal	equ	0xF8	;should be 0xF8 if crystal freq. is perfect
			;adjust up and down for 0.004s increments

;timeout
toDefHrs    equ         0x01
toDefMin    equ         0x00

;register assignments
mode	equ	0x20	;status flags
mTime	equ	0x21	;master timer
command	equ	0x22	;parsed command, 0=no command
paramH	equ	0x23	;parameter decoding
paramL	equ	0x24
power	equ	0x25	;source power setting 0-100
wTemp	equ	0x2A	;status save
sTemp	equ	0x2B
temp1	equ	0x2C	;temp use registers
temp2	equ	0x2D
temp3	equ	0x2E
temp4       equ         0x2F
comPtr	equ	0x26	;serial recieve buffer pointer
comBuf	equ	0x40	;serial recieve buffer

;time
toHrs	equ	0x30
toMin	equ	0x31
sec	equ	0x32

;aliases
temp	equ	temp1
comFlag	equ	mode
resHi	equ	temp1
resLo	equ	temp2
tempH	equ	temp1
tempL	equ	temp2
tblPtrH	equ	temp3
tblPtrL	equ	temp4
bufPtr	equ	temp2
char	equ	temp1
count	equ	temp4

;flags in mode
srcStat     equ         0x0         ;is source on? 1=On
comPend	equ	0x6	;serial command present in buffer
comCmd	equ	0x7	;command parsed for execution

;GPIO pin assignments
;port A
iPort	equ	PORTA
iDir	equ	TRISA
iRead	equ	0x0

;port B
ctrlPort	equ	PORTB
ctrlDir	equ	TRISB
indLED	equ	0x1
srcEnb	equ	0x2

;port C
comPort     equ         PORTC
comDir	equ	TRISC
comCTS	equ	0x0
comRTS	equ	0x1
comTX	equ	0x6
comRX	equ	0x7
daPort	equ	PORTC	;LTC1662 port control pins
daDir	equ	TRISC
daLoad	equ	0x02


;------------------------------------------------------------------------------
;  reset
;  - reset and initialize a PIC microcontroller
;
;  10 Nov 2008  A. Cooper
;  - Initial version
;
;  Input:  none
;  Output: none
;
;------------------------------------------------------------------------------

	ORG	0x0000
Reset	
	pagesel	Init	;in case coming from codeLoader
	goto	Init	;reset vector


;----------------------------------------------------------------------------
;  Interrupt
;
;  - interript service routine
;  - handles a TMR2 interrupt only
;	

	ORG	0x0004
Interrupt	movwf	wTemp	;save state
	movf	STATUS,w
	movwf	sTemp
	banksel	PIR1	;check and reset timer 2 overflow bit
	btfss	PIR1,TMR2IF
	goto	IntEnd
	bcf	PIR1,TMR2IF
	incf	mTime,f	;increment master timebase
IntEnd	movf	sTemp,w	;restore state
	movwf	STATUS
	swapf	wTemp,f
	swapf	wTemp,w
	retfie	


;----------------------------------------------------------------------------
;  Includes
;
;  - the bulk of the code is here...
;	

	include	"host.asm"	;message send routine
	include	"sourceCmd.asm" ;command execution
	include	"serial.asm" ;serial routines
	include	"misc.asm"	;miscellaneous routines
	include	"parse.asm"	;parse command inputs
	include	"LTC1662.asm" ;LTC1662 DAC routines
	include	"adc.asm"	;internal adc support


;----------------------------------------------------------------------------
;  Init
;
;  - set mode contol registers and initialize variables
;	
Init
	banksel	TMR0	;set for bank 2
	clrf	TMR0	;clear timer 0
	clrf	INTCON	;disable all interrupts
	clrwdt		
	banksel	OPTION_REG	;set OPTION_REG
	movlw	B'10001111'	;PORTB pullups disabled
			;interrupt edge falling
			;TMR0 clock source internal
			;TMRO increment on low to high
			;prescaler assigned to WDT
			;prescaler set for 1:64
	movwf	OPTION_REG
            banksel     ADCON1      ;set port A to all digital
            movlw       0x07
            movwf       ADCON1
	banksel	mode
	clrf	mode	;clear/set control variables
	clrf	command
	clrf	comPtr
	clrf	sec
            clrf        power
	movlw	toDefHrs	;set reset timeout
	movwf	toHrs
	movlw	toDefMin
	movwf	toMin
            banksel     ctrlPort     ;set source enable bit off
            bcf         ctrlPort,srcEnb
            banksel     ctrlDir      ;insure source control bit is output
            bcf         ctrlDir,srcEnb
	pagesel	ComInit	;start UART
	call	ComInit
	pagesel	DAInit	;initialize SPI port for use with DAC
	call	DAInit
	banksel	TMR2	;setup timer 2 for main 10Hz timebase
	clrf	TMR2
	movlw	B'00011101'	;Pre /4, post /4
	movwf	T2CON
	banksel	PR2	;period to 250
	movlw	0xFA
	movwf	PR2
	banksel	INTCON
	clrf	INTCON	;disable all interrupts		
	banksel	PIE1	;allow timer 2 interrupt only
	movlw	B'00000010'
	movwf	PIE1
	movlw	B'11000000'	;allow global interrupts
	movwf	INTCON
	pagesel	Del100ms	;delay 200ms
	call	Del100ms
	pagesel	HostMsg
	movlw	HMLine	;send a couple blank lines
	call	HostMsg
	movlw	HMLine
	call	HostMsg
	movlw	HMRevision	;send ID text
	call	HostMsg
	movlw	HMLine
	call	HostMsg
	movlw	HMReset	;send reset text
	call	HostMsg
	movlw	HMPrompt	;send prompt
	call	HostMsg
	ComLEDOff		;set LED's
	

;----------------------------------------------------------------------------
;  Main
;
;  - primary operation control loop
;

Main	clrwdt                  ;clear watchdog
	pagesel	MakeTime	;update real time clock
	call	MakeTime
            pagesel     TOCheck     ;check timeout
            call        TOCheck
            pagesel     ErrCheck    ;check possible error conditions
            call        ErrCheck
	pageSel	ComCheck	;get command if available
	call	ComCheck
	pagesel	Main
	banksel	mode
	btfss	mode,comPend
	 goto	Main
MainP	
	pagesel	Parse	;parse the recieved command
	call	Parse
	pagesel	Main
	banksel	mode
	btfss	mode,comCmd
	 goto	Main
	pagesel	SourceCmd	;execute the command
	call	SourceCmd
	pagesel	HostMsg
	movlw	HMPrompt
	call	HostMsg
	pagesel	Del10ms
	call	Del10ms
	ComLEDOff
	pagesel	Main
	goto	Main


;----------------------------------------------------------------------------
;  TOCheck
;
;  - check for timeout
;
TOCheck     
	banksel	mode
	btfsc       mode,srcStat ;is the source on?
             return
            movf        toHrs,f       ;check timer for zero
            btfss       STATUS,Z
             return
            movf        toMin,f
            btfss       STATUS,Z
             return
            movlw       HMLine      ;timeout has occured
            pagesel     HostMsg
            call        HostMsg
            movlw       HMTimeout
            pagesel     HostMsg
            call        HostMsg
	pagesel	Init
	goto	Init


;----------------------------------------------------------------------------
;  ErrChecks
;
;  - double check for any illegal conditions
;  - perform a full reset if any occur
;
ErrCheck    
            banksel     toHrs	;check for illegal timeout
            movf        toHrs,w
            sublw       toDefHrs
            btfss       STATUS,C
             goto       Init
            movf        toMin,w
            sublw       0x3B
            btfss       STATUS,C
	 goto       Init
            movf        power,w	;check for illegal settings
	btfsc	STATUS,Z
	 return
            sublw       0x09
            btfsc       STATUS,C
             goto       Init
            movf        power,w
            sublw       0x66
            btfss       STATUS,C
             goto       Init
            return


;------------------------------------------------------------------------------
;  the rest of the program...
;
	
	org	0x0FFF
	goto	Init	;reset at the end of memory	

	END
;------------------------------------------------------------------------------
