;----------------------------------------------------------------------------
;
;  serial.asm
;
;  Andrew Cooper
;  http://www.darkerview.com
;  Copyright 2003-2012 all rights reserved
;
;  - handle serial communication with a host
;  - ComInit must be called to setup the serial port, port configuration
;  is hard coded in this routine (buad rate, bits, etc.)
;  - check the serial port for input, if present add to buffer
;  - when <CR> recieved set the flag comCmd
;  - requires 16 bytes to be reserved at comBuf
;  - requires the locations comPtr and comFlags reserved in the
;  multibank mapped area, as well as the constant comPend
; 
;	;serial communications buffer
;	comBuf	equ	0x60	;16 byte comm. buffer
;	;communications flags in comFlag
;	comPend	equ	0x0	;pending command in buffer
; 	;multi-bank mapped registers
;	comChar	equ	0x73	;serial recieved char
;	comFlag	equ	0x74	;serial recieved flag
;           ;pin assignments
;           CTS         equ         0x4         ;pin used for CTS
;           RTS         equ         0x5         ;pin used for RTS
;           comPort     equ         PORTC       ;port used for CTS and RTS
;
;  Andrew Cooper Silicon Owl
;  Copyright 2003 all rights reserved
;
;  23 Nov 2003 A. Cooper
;  - initial version
;  4 Feb 2012 A. Cooper
;  - modified for use on the PIC16F873 (must banksel all RAM access)
;  - converted LED On/Off to macros
;
;
;----------------------------------------------------------------------------

;----------------------------------------------------------------------------
;  ComLEDOn
;
;  - Turn on the commincations indicator
;
ComLEDOn	macro
	banksel	ctrlDir	;set the pin as output
	bcf	ctrlDir,indLED
	banksel	ctrlPort
	bsf	ctrlPort,indLED
	endm


;----------------------------------------------------------------------------
;  ComLEDOff
;
;  - Turn off the commincations indicator
;
ComLEDOff	macro
	banksel	ctrlDir	;set the pin as output
	bcf	ctrlDir,indLED
	banksel	ctrlPort
	bcf	ctrlPort,indLED
	endm


;----------------------------------------------------------------------------
;  ComInit
;
;  - Setup serial port for use
;  - port configuration hardcoded
;      mode:  async
;      buad:  9600 (w/4MHz clock)
;      bits:  8
;      start: 1
;      stop:  1
;
ComInit	
	banksel	TXSTA	
	movlw	B'00100100' ;set transmission control register
			;asynchronous mode
			;8 bit transmission
			;transmit enabled
			;high speed baud rate generation
	movwf	TXSTA
	movlw	0x19	;set baud rate generator for buad rate
	movwf	SPBRG
	banksel	comDir
	bcf	comDir,comTX ;output for TX pin
	bsf	comDir,comRX ;input for RX pin
	bcf	comDir,comCTS ;output for CTS pin
	bsf	comDir,comRTS ;input on RTS pin
	banksel	RCSTA
	movlw	B'10010000' ;set recieve control register
			;serial port enabled
			;8 bit reception
			;continuous receive mode
			;disable address detection
	movwf	RCSTA	;enable serial transceiver
            ComLEDOn    	;flash LED
	return


;----------------------------------------------------------------------------
;  ComSend & ComDSend
;
;  - Send a byte on the serial port
;  - Byte to be sent in W on entry
;  - Assumes ComInit run before call to init the port 
;  - ComDSend adds 0x30 to decimal vales to convert to ASCII
;
ComDSend	addlw	0x30	;convert decimal to ASCII
ComSend	
	banksel	PIR1
ComSWait	btfss	PIR1,TXIF	;check transmission register for empty
	 goto	ComSWait
	banksel	TXREG
	movwf	TXREG	;send the byte
	return


;----------------------------------------------------------------------------
;  ComCheck
;
;  - Handle serial recieve
;
ComCheck	
	banksel	comFlag
	bcf	comFlag,comPend
            banksel     comPort     ;clear CTS
            bcf         comPort,comCTS
	banksel	RCSTA	;clear hardware overrun error
	btfsc	RCSTA,OERR
	bcf	RCSTA,CREN
	bsf	RCSTA,CREN
	banksel	PIR1
	btfss	PIR1,5	;check serial recieved flag
	 return
	ComLEDOn
	movlw	comBuf	;setup buffer write
	addwf	comPtr,w
	movwf	FSR
	bcf	STATUS,IRP
	movf	RCREG,w	;save recieved character
	movwf	INDF
	pagesel	ComSend
	call	ComSend	;echo the recieved character
	banksel	comPtr
	incf	comPtr,f	;increment pointer
	btfsc	comPtr,4	;check for buffer overflow
	 clrf	comPtr	;reset on overflow
	;sublw	0x0A	;check for <Newline>
	sublw	0x0D	;check for <CR>
	btfss	STATUS,Z
	 return
	movlw	0x0A	;send <NewLine>
	pagesel	ComSend
	call	ComSend
	banksel	comPtr
	clrf	comPtr	;reset buffer pointer
	bsf	comFlag,comPend
            banksel     comPort
            bsf         comPort,comCTS ;set CTS while processing command
	return

; end of serial.asm
;----------------------------------------------------------------------------
